@@ -22,6 +22,7 @@ import os
 import gtk
 import gtk.glade
 import gobject
+import pango
 
 # project
 import paths
@@ -44,7 +45,7 @@ for sourceview in "gtksourceview sourceview".split():
     except ImportError:
         pass
 
-version = "1.1.5"
+version = "1.2"
 
 # magic developer switch, changes some behaviour
 developer = 0
@@ -84,7 +85,7 @@ class NewDocDialog(gnomeglade.Component):
         self.entrylists[page][ not button.get_active() ].gtk_entry().grab_focus()
 
     def on_response(self, dialog, arg):
-        if arg==gtk.RESPONSE_OK:
+        if arg != gtk.RESPONSE_CANCEL:
             page = self.notebook.get_current_page()
             paths = [ e.get_full_path(0) or "" for e in self.entrylists[page] ]
             if page < 2 and not self.three_way_compare[page].get_active():
@@ -259,14 +260,7 @@ class PreferencesDialog(gnomeglade.Component):
         self.entry_text_codecs.set_text( self.prefs.text_codecs )
         self._map_widgets_into_lists( ["save_encoding"] )
         self.save_encoding[self.prefs.save_encoding].set_active(1)
-        # cvs
-        self.cvs_quiet_check.set_active( self.prefs.cvs_quiet )
-        self.cvs_compression_check.set_active( self.prefs.cvs_compression )
-        self.cvs_compression_value_spin.set_value( self.prefs.cvs_compression_value )
-        self.cvs_ignore_cvsrc_check.set_active( self.prefs.cvs_ignore_cvsrc )
-        self.cvs_binary_fileentry.set_filename( self.prefs.cvs_binary )
-        self.cvs_create_missing_check.set_active( self.prefs.cvs_create_missing )
-        self.cvs_prune_empty_check.set_active( self.prefs.cvs_prune_empty )
+        self.treeview.set_cursor(0)
     #
     # treeview
     #
@@ -327,23 +321,6 @@ class PreferencesDialog(gnomeglade.Component):
     def on_save_encoding_toggled(self, radio):
         if radio.get_active():
             self.prefs.save_encoding = self.save_encoding.index(radio)
-    #
-    # cvs
-    #
-    def on_cvs_quiet_toggled(self, toggle):
-        self.prefs.cvs_quiet = toggle.get_active()
-    def on_cvs_compression_toggled(self, toggle):
-        self.prefs.cvs_compression = toggle.get_active()
-    def on_cvs_compression_value_changed(self, spin):
-        self.prefs.cvs_compression_value = int(spin.get_value())
-    def on_cvs_ignore_cvsrc_toggled(self, toggle):
-        self.prefs.cvs_ignore_cvsrc = toggle.get_active()
-    def on_cvs_binary_activate(self, fileentry):
-        self.prefs.cvs_binary = fileentry.gtk_entry().get_text()
-    def on_cvs_create_missing_toggled(self, toggle):
-        self.prefs.cvs_create_missing = toggle.get_active()
-    def on_cvs_prune_empty_toggled(self, toggle):
-        self.prefs.cvs_prune_empty = toggle.get_active()
     def on_response(self, dialog, arg):
         if arg==gtk.RESPONSE_CLOSE:
             self.prefs.text_codecs = self.entry_text_codecs.get_property("text")
@@ -377,25 +354,54 @@ class MeldStatusBar(object):
 #
 ################################################################################
 class NotebookLabel(gtk.HBox):
+    tab_width_in_chars = 30
+
+    def __init__(self, iconname, text, onclose):
+        gtk.HBox.__init__(self, False, 4)
+
+        label = gtk.Label(text)
+        # FIXME: ideally, we would use custom ellipsization that ellipsized the
+        # two paths separately, but that requires significant changes to label
+        # generation in many different parts of the code
+        label.set_ellipsize(pango.ELLIPSIZE_MIDDLE);
+        label.set_single_line_mode(True)
+        label.set_alignment(0.0, 0.5)
+        label.set_padding(0, 0)
+
+        context = self.get_pango_context()
+        metrics = context.get_metrics(self.style.font_desc, context.get_language())
+        char_width = metrics.get_approximate_digit_width()
+        (w, h) = gtk.icon_size_lookup_for_settings (self.get_settings(), gtk.ICON_SIZE_MENU)
+        self.set_size_request(self.tab_width_in_chars * pango.PIXELS(char_width) + 2 * w, -1)
+
+        button = gtk.Button()
+        button.set_relief(gtk.RELIEF_NONE)
+        button.set_focus_on_click(False)
+        image = gtk.image_new_from_stock(gtk.STOCK_CLOSE, gtk.ICON_SIZE_MENU)
+        image.set_tooltip_text(_("Close tab"))
+        button.add(image)
+        button.set_name("meld-tab-close-button")
+        button.set_size_request(w + 2, h + 2)
+        button.connect("clicked", onclose)
 
-    def __init__(self, iconname, text="", onclose=None):
-        gtk.HBox.__init__(self)
-        self.label = gtk.Label(text)
-        self.button = gtk.Button()
-        self.button.set_relief(gtk.RELIEF_NONE)
-        image = gtk.Image()
-        image.set_from_stock( gtk.STOCK_CLOSE, gtk.ICON_SIZE_MENU)
-        image.set_size_request( 10,10 ) #TODO font height
-        self.button.add( image )
         icon = gtk.Image()
         icon.set_from_file( paths.share_dir("glade2/pixmaps/%s" % iconname) )
-        icon.set_from_pixbuf( icon.get_pixbuf().scale_simple(15, 15, 2) ) #TODO stock image
-        self.pack_start( icon )
-        self.pack_start( self.label )
-        self.pack_start( self.button, expand=0 )
+        icon.set_from_pixbuf(icon.get_pixbuf().scale_simple(16, 16, 2)) #TODO stock image
+
+        self.pack_start(icon, expand=False)
+        self.pack_start(label)
+        self.pack_start(button, expand=False)
+        self.set_tooltip_text(text)
         self.show_all()
-        if onclose:
-            self.button.connect("clicked", onclose)
+
+        self.__label = label
+
+    def get_label_text(self):
+        return self.__label.get_text()
+
+    def set_label_text(self, text):
+        self.__label.set_text(text)
+        self.set_tooltip_text(text)
 
 ################################################################################
 #
@@ -420,13 +426,6 @@ class MeldPreferences(prefs.Preferences):
         "draw_style": prefs.Value(prefs.INT,2),
         "toolbar_style": prefs.Value(prefs.INT,0),
         "ignore_symlinks": prefs.Value(prefs.BOOL,0),
-        "cvs_quiet": prefs.Value(prefs.BOOL, 1),
-        "cvs_compression": prefs.Value(prefs.BOOL, 1),
-        "cvs_compression_value": prefs.Value(prefs.INT, 3),
-        "cvs_ignore_cvsrc": prefs.Value(prefs.BOOL, 0),
-        "cvs_binary": prefs.Value(prefs.STRING, "/usr/bin/cvs"),
-        "cvs_create_missing": prefs.Value(prefs.BOOL, 1),
-        "cvs_prune_empty": prefs.Value(prefs.BOOL, 1),
         "vc_console_visible": prefs.Value(prefs.BOOL, 0),
         "color_delete_bg" : prefs.Value(prefs.STRING, "DarkSeaGreen1"),
         "color_delete_fg" : prefs.Value(prefs.STRING, "Red"),
@@ -442,13 +441,13 @@ class MeldPreferences(prefs.Preferences):
             #TRANSLATORS: translate this string ONLY to the first "\t", leave it and the following parts intact
             _("Backups\t1\t#*# .#* ~* *~ *.{orig,bak,swp}\n") + \
             #TRANSLATORS: translate this string ONLY to the first "\t", leave it and the following parts intact
-            _("Version Control\t1\tCVS .svn MT [{]arch[}] .arch-ids .arch-inventory\n") + \
+            _("Version Control\t1\tCVS .svn MT [{]arch[}] .arch-ids .arch-inventory RCS\n") + \
             #TRANSLATORS: translate this string ONLY to the first "\t", leave it and the following parts intact
             _("Binaries\t1\t*.{pyc,a,obj,o,so,la,lib,dll}\n") + \
             #TRANSLATORS: translate this string ONLY to the first "\t", leave it and the following parts intact
             _("Media\t0\t*.{jpg,gif,png,wav,mp3,ogg,xcf,xpm}")),
             #TRANSLATORS: translate this string ONLY to the first "\t", leave it and the following parts intact
-            "regexes" : prefs.Value(prefs.STRING, _("CVS keywords\t0\t\$\\w+(:[^\\n$]+)?\$\n") + \
+        "regexes" : prefs.Value(prefs.STRING, _("CVS keywords\t0\t\$\\w+(:[^\\n$]+)?\$\n") + \
             #TRANSLATORS: translate this string ONLY to the first "\t", leave it and the following parts intact
             _("C++ comment\t0\t//.*\n") + \
             #TRANSLATORS: translate this string ONLY to the first "\t", leave it and the following parts intact
@@ -465,22 +464,6 @@ class MeldPreferences(prefs.Preferences):
     def __init__(self):
         prefs.Preferences.__init__(self, "/apps/meld", self.defaults)
 
-    def get_cvs_command(self, op=None):
-        cmd = [self.cvs_binary]
-        if self.cvs_quiet:
-            cmd.append("-q")
-        if self.cvs_compression:
-            cmd.append("-z%i" % self.cvs_compression_value)
-        if self.cvs_ignore_cvsrc:
-            cmd.append("-f")
-        if op:
-            cmd.append(op)
-            if op == "update":
-                if self.cvs_create_missing:
-                    cmd.append("-d")
-                if self.cvs_prune_empty:
-                    cmd.append("-P")
-        return cmd
     def get_current_font(self):
         if self.use_custom_font:
             return self.custom_font
@@ -530,6 +513,14 @@ class MeldApp(gnomeglade.GnomeApp):
     # init
     #
     def __init__(self):
+        gtk.rc_parse_string ("style \"meld-tab-close-button-style\"\n"
+                     "{\n"
+                         "GtkWidget::focus-padding = 0\n"
+                         "GtkWidget::focus-line-width = 0\n"
+                         "xthickness = 0\n"
+                         "ythickness = 0\n"
+                     "}\n"
+                     "widget \"*.meld-tab-close-button\" style \"meld-tab-close-button-style\"")
         gladefile = paths.share_dir("glade2/meldapp.glade")
         gnomeglade.GnomeApp.__init__(self, "meld", version, gladefile, "meldapp")
         self._map_widgets_into_lists( "settings_drawstyle".split() )
@@ -561,11 +552,13 @@ class MeldApp(gnomeglade.GnomeApp):
             self.statusbar.task_progress.set_fraction(0)
         if self.scheduler.tasks_pending():
             self.toolbar_stop.set_sensitive(1)
+            self.menu_view_stop.set_sensitive(1)
             return 1
         else:
             self.statusbar.set_task_status("")
             self.idle_hooked = 0
             self.toolbar_stop.set_sensitive(0)
+            self.menu_view_stop.set_sensitive(0)
             return 0
 
     def on_scheduler_runnable(self, sched):
@@ -589,14 +582,14 @@ class MeldApp(gnomeglade.GnomeApp):
         self.button_undo.set_sensitive(newseq.can_undo())
         self.button_redo.set_sensitive(newseq.can_redo())
         nbl = self.notebook.get_tab_label( newdoc.widget )
-        self.widget.set_title( nbl.label.get_text() + " - Meld")
+        self.widget.set_title(nbl.get_label_text() + " - Meld")
         self.statusbar.set_doc_status("")
         newdoc.on_switch_event()
         self.scheduler.add_task( newdoc.scheduler )
 
     def on_notebook_label_changed(self, component, text):
         nbl = self.notebook.get_tab_label( component.widget )
-        nbl.label.set_text(text)
+        nbl.set_label_text(text)
         self.widget.set_title(text + " - Meld")
         self.notebook.child_set_property(component.widget, "menu-label", text)
 
@@ -729,7 +722,7 @@ class MeldApp(gnomeglade.GnomeApp):
 
     def try_remove_page(self, page):
         "See if a page will allow itself to be removed"
-        if page.on_delete_event() == gtk.RESPONSE_OK:
+        if page.on_delete_event() != gtk.RESPONSE_CANCEL:
             self.scheduler.remove_scheduler( page.scheduler )
             i = self.notebook.page_num( page.widget )
             assert(i>=0)
@@ -744,7 +737,7 @@ class MeldApp(gnomeglade.GnomeApp):
                 page.on_file_changed(filename)
 
     def _append_page(self, page, icon):
-        nbl = NotebookLabel(icon, onclose=lambda b: self.try_remove_page(page))
+        nbl = NotebookLabel(icon, "", lambda b: self.try_remove_page(page))
         self.notebook.append_page( page.widget, nbl)
         self.notebook.set_current_page( self.notebook.page_num(page.widget) )
         self.scheduler.add_scheduler(page.scheduler)
